#ifdef MAC
#include <OpenCL/cl.h>
#else  
#include <CL/cl.h>
#endif
#include "opencl-errors.hpp"

#ifndef OPENCL_H
#define OPENCL_H

#include <iostream>

static inline void check_opencl_error(const char* opencl_op,
      const char* source, unsigned int line, cl_int error) {
   if (error != CL_SUCCESS) {
      const char* msg = opencl_error_to_string(error);
      std::cerr << opencl_op << " at " << source << ":" << line
	 << " failed: " << (msg? msg: "undefined error") << std::endl;
      exit(1);
   }
}

#define CHECK_OPENCL(opname, ...) \
   check_opencl_error(#opname, __FILE__, __LINE__, opname(__VA_ARGS__))
#define CHECK_OPENCL_ERRCODE(opname, errcode) \
   check_opencl_error(opname, __FILE__, __LINE__, errcode)

#endif
