#pragma OPENCL EXTENSION cl_khr_fp64: enable

// to be integrated function
double f(double x) {
   return 4 / (1 + x*x);
}

// numerical integration according to the Simpson rule
// for f over the i-th subinterval of [a,b]
__kernel void simpson(double a, double b, __global double* sums) {
   const int N = get_global_size(0);
   const unsigned int i = get_global_id(0);
   double xleft = a + (b - a) / N * i;
   double xright = xleft + (b - a) / N;
   double xmid = (xleft + xright) / 2;
   sums[i] = (xright - xleft) / 6 * (f(xleft) + 4 * f(xmid) + f(xright));
}
