#include <math.h>
#include <iomanip>
#include <iostream>
#include <sstream>
#include "opencl.hpp"
#include "simpson-kernel.hpp"

char* cmdname;
void usage() {
   std::cerr << "Usage: " << cmdname << " [# intervals]" << std::endl;
   exit(1);
}

int main(int argc, char** argv) {
   const double a = 0;
   const double b = 1;
   unsigned int N = 8192;

   cmdname = *argv++; --argc;
   if (argc > 0) {
      std::istringstream arg(*argv++); --argc;
      if (!(arg >> N) || N <= 0) usage();
   }
   if (argc > 0) usage();

   /* open context for the first GPU device that we find */
   cl_platform_id platform;
   CHECK_OPENCL(clGetPlatformIDs, 1, &platform, 0);
   cl_device_id device_id;
   CHECK_OPENCL(clGetDeviceIDs, platform, CL_DEVICE_TYPE_GPU, 1, &device_id, 0);
   cl_int errcode = 0;
   cl_context context = clCreateContext(/* properties = */ 0,
      /* num_devices = */ 1, /* devices = */ &device_id, /* pfn_notify = */ 0,
      /* user_data = */ 0, &errcode);
   CHECK_OPENCL_ERRCODE("clCreateContext", errcode);

   const unsigned char* binary = simpson_kernel;
   cl_program program = clCreateProgramWithBinary(context,
      1, &device_id, &simpson_kernel_size, &binary,
      0, &errcode);
   CHECK_OPENCL_ERRCODE("clCreateProgramWithBinary", errcode);
   CHECK_OPENCL(clBuildProgram, program, 1, &device_id, 0, 0, 0);
   cl_kernel kernel = clCreateKernel(program, "simpson", &errcode);
   CHECK_OPENCL_ERRCODE("clCreateKernel", errcode);
   cl_command_queue queue = clCreateCommandQueue(context, device_id,
      0, &errcode);
   CHECK_OPENCL_ERRCODE("clCreateCommandQueue", errcode);

   cl_mem sums = clCreateBuffer(context, CL_MEM_WRITE_ONLY,
      sizeof(double) * N, 0, &errcode);
   CHECK_OPENCL_ERRCODE("clCreateBuffer", errcode);
   CHECK_OPENCL(clSetKernelArg, kernel, 0, sizeof(double), &a);
   CHECK_OPENCL(clSetKernelArg, kernel, 1, sizeof(double), &b);
   CHECK_OPENCL(clSetKernelArg, kernel, 2, sizeof(cl_mem), &sums);

   const size_t work_size = N;
   CHECK_OPENCL(clEnqueueNDRangeKernel, queue, kernel,
      /* work_dim = */ 1, /* global work offset = */ 0,
      /* global work size = */ &work_size,
      /* local work size = */ 0, /* determined by OpenCL implementation */
      /* num_events_in_wait_list = */ 0, /* event_wait_list = */ 0,
      /* event = */ 0);
   double* host_sums = new double[N];
   CHECK_OPENCL(clEnqueueReadBuffer,queue, sums,
      /* blocking read = */ CL_TRUE,
      /* offset = */ 0,
      /* cb = */ sizeof(double) * N,
      /* ptr = */ host_sums,
      /* num_events_in_wait_list = */ 0, /* event_wait_list = */ 0,
      /* event = */ 0);
   double sum = 0;
   for (int i = 0; i < N; ++i) {
      sum += host_sums[i];
   }
   std::cout << std::setprecision(14) << sum << std::endl;
   std::cout << std::setprecision(14) << M_PI << std::endl;
}
